@@ -42,25 +42,6 @@
 
 (define sleep (@ (fibers) sleep))
 
-(define-syntax-rule (unwind-protect body ... conclude)
-  "Evaluate BODY... and return its result(s), but always evaluate CONCLUDE
-before leaving, even if an exception is raised.
-
-This is *not* implemented with 'dynamic-wind' in order to play well with
-delimited continuations and fibers."
-  (let ((conclusion (lambda () conclude)))
-    (catch #t
-      (lambda ()
-        (call-with-values
-            (lambda ()
-              body ...)
-          (lambda results
-            (conclusion)
-            (apply values results))))
-      (lambda args
-        (conclusion)
-        (apply throw args)))))
-
 (define (call-with-server-socket file-name proc)
   "Call PROC, passing it a listening socket at FILE-NAME and deleting the
 socket file at FILE-NAME upon exit of PROC.  Return the values of PROC."
@@ -76,17 +57,20 @@ socket file at FILE-NAME upon exit of PROC.  Return the values of PROC."
                      ;; Stop services that were started from the config file
                      ;; and quit.
                      (stop-service root-service)))))))
-    (unwind-protect (proc sock)
-                    (begin
-                      (close sock)
-                      (catch-system-error (delete-file file-name))))))
+    (and sock
+         (catch #t
+           (lambda ()
+             (proc sock))
+           (lambda args
+             (close sock)
+             (apply throw args))))))
 
 (define (maybe-signal-port signals)
   "Return a signal port for SIGNALS, using 'signalfd' on GNU/Linux, or #f if
 that is not supported."
   (catch 'system-error
     (lambda ()
-      (let ((port (non-blocking-port (signalfd -1 signals))))
+      (let ((port (signalfd -1 signals (logior SFD_CLOEXEC SFD_NONBLOCK))))
         ;; As per the signalfd(2) man page, block SIGNALS.  The tricky bit is
         ;; that SIGNALS must be blocked for all the threads; new threads will
         ;; inherit the signal mask, but we must ensure that neither Guile's
@@ -155,9 +139,56 @@ already ~a threads running, disabling 'signalfd' support")
            (fcntl fd F_SETFD (logior FD_CLOEXEC flags)))))
       (loop (+ fd 1)))))
 
+(cond-expand
+ ((not guile-3)
+  (define exception-with-kind-and-args?
+    (const #f)))
+ (else
+  (define exception-with-kind-and-args?
+    (exception-predicate &exception-with-kind-and-args))))
+
+(define (configuration-file-loader file)
+  "Return a thunk that loads @var{file}, the user's configuration file."
+  (define (failure)
+    (report-error
+     (l10n "~s: exception thrown while loading configuration file~%")
+     file)
+    #f)
+
+  (define (handle-key-and-args-exception key args)
+    (match key
+      ('system-error
+       (local-output
+        (l10n "While loading configuration file '~a': ~a")
+        file (strerror (system-error-errno (cons key args)))))
+      (_
+       (local-output
+        (l10n "Uncaught exception while loading configuration file '~a': ~s")
+        file (cons key args)))))
+
+  (lambda ()
+    (guard (c ((action-runtime-error? c)
+               (local-output (l10n "action '~a' on service '~a' failed: ~s")
+                             (action-runtime-error-action c)
+                             (action-runtime-error-service c)
+                             (cons (action-runtime-error-key c)
+                                   (action-runtime-error-arguments c)))
+               (failure))
+              ((exception-with-kind-and-args? c)
+               (handle-key-and-args-exception
+                (exception-kind c) (exception-args c))
+               (failure)))
+      (catch #t
+        (lambda ()
+          (load-in-user-module file)
+          (local-output (l10n "Configuration successfully loaded from '~a'.")
+                        file))
+        (lambda (key . args)                      ;for Guile 2.2
+          (handle-key-and-args-exception key args))))))
+
 (define* (run-daemon #:key (config-file (default-config-file))
                      socket-file pid-file signal-port poll-services?)
-  (define signal-handler
+  (define (signal-thunk signal-port)
     ;; Thunk that waits for signals (particularly SIGCHLD) and handles them.
     (if signal-port
         (lambda ()
@@ -181,15 +212,29 @@ already ~a threads running, disabling 'signalfd' support")
   ;; mark them all as FD_CLOEXEC so child processes do not inherit them.
   (mark-as-close-on-exec)
 
-  ;; This _must_ succeed.  (We could also put the `catch' around
-  ;; `main', but it is often useful to get the backtrace, and
-  ;; `caught-error' does not do this yet.)
-  (catch #t
-    (lambda ()
-      (load-in-user-module (or config-file (default-config-file))))
-    (lambda (key . args)
-      (caught-error key args)
-      (quit 1)))
+  (when signal-port
+    ;; When the 'daemonize' action is invoked, open a new signal port in the
+    ;; child process and spawn a new fiber reading it.  This required because
+    ;; after fork(2), epoll_wait(2), which is used by Fibers, does not signal
+    ;; that the signal FD is ready for reading--see the signalfd(2) man page.
+    (add-hook! %post-daemonize-hook
+               (lambda ()
+                 (local-output (l10n "Restarting signal handler."))
+                 (close-port signal-port)
+                 (spawn-fiber
+                  (essential-task-thunk
+                   'signal-handler
+                   (signal-thunk (maybe-signal-port %precious-signals)))))))
+
+  ;; Spawn a signal handling fiber.
+  (spawn-fiber
+   (essential-task-thunk 'signal-handler
+                         (signal-thunk signal-port)))
+
+  ;; Load CONFIG-FILE in another fiber.  If loading fails, report it but keep
+  ;; going: the user can use 'herd load root' with a new config file if
+  ;; needed.
+  (spawn-fiber (configuration-file-loader config-file))
 
   ;; Ignore SIGPIPE so that we don't die if a client closes the connection
   ;; prematurely.
@@ -212,10 +257,6 @@ already ~a threads running, disabling 'signalfd' support")
            (#t (display (getpid)))
            (_  #t))
 
-         ;; Spawn a signal handling fiber.
-         (spawn-fiber
-          (essential-task-thunk 'signal-handler signal-handler))
-
          ;; Enter some sort of a REPL for commands.
          (let next-command ()
            (match (accept sock (logior SOCK_NONBLOCK SOCK_CLOEXEC))
@@ -415,29 +456,22 @@ fork in the child process."
              (register-services (list root-service))
              (start-service root-service)
 
-             (catch 'quit
-               (lambda ()
-                 (with-process-monitor
-                   ;; Replace the default 'system*' binding with one that
-                   ;; cooperates instead of blocking on 'waitpid'.  Replace
-                   ;; 'primitive-load' (in C as of 3.0.9) with one that does
-                   ;; not introduce a continuation barrier.
-                   (replace-core-bindings!
-                    (system* (lambda command
-                               (spawn-command command)))
-                    (system spawn-shell-command)
-                    (primitive-load primitive-load*))
+             (with-process-monitor
+               ;; Replace the default 'system*' binding with one that
+               ;; cooperates instead of blocking on 'waitpid'.  Replace
+               ;; 'primitive-load' (in C as of 3.0.9) with one that does
+               ;; not introduce a continuation barrier.
+               (replace-core-bindings!
+                (system* (lambda command
+                           (spawn-command command)))
+                (system spawn-shell-command)
+                (primitive-load primitive-load*))
 
-                   (run-daemon #:socket-file socket-file
-                               #:config-file config-file
-                               #:pid-file pid-file
-                               #:signal-port signal-port
-                               #:poll-services? poll-services?)))
-               (case-lambda
-                 ((key value . _)
-                  (primitive-exit value))
-                 ((key)
-                  (primitive-exit 0))))))
+               (run-daemon #:socket-file socket-file
+                           #:config-file (or config-file (default-config-file))
+                           #:pid-file pid-file
+                           #:signal-port signal-port
+                           #:poll-services? poll-services?))))
          #:parallelism 1                          ;don't create POSIX threads
          #:hz 0)))))       ;disable preemption, which would require POSIX threads
 
