@@ -236,7 +236,10 @@ into a @code{live-service} record."
 
   (if (live-service-enabled? service)
       (format #t (l10n "  It is enabled.~%"))
-      (format #t (highlight/warn (l10n "  It is disabled.~%"))))
+      (let ((highlight (if (null? (live-service-last-respawns service))
+                           highlight/warn
+                           highlight/error)))
+        (format #t (highlight (l10n "  It is disabled.~%")))))
   (format #t (l10n "  Provides ~a.~%") (live-service-provision service))
   (format #t (l10n "  Requires ~a.~%") (live-service-requirement service))
   (if (live-service-respawn? service)
@@ -478,6 +481,18 @@ the daemon via SOCKET-FILE."
                 ('messages messages))
         (for-each display-line messages)
         (report-command-error error)
+
+        ;; Did the user swap ACTION and SERVICE?
+        (match (list action service)
+          ((_ (or 'start 'stop 'status 'doc))
+           (report-error (l10n "Did you mean 'herd ~a ~a'?")
+                         service action))
+          ((root (or 'help 'halt 'power-off 'load 'eval 'unload 'reload
+                     'daemonize 'restart))
+           (report-error (l10n "Did you mean 'herd ~a ~a'?")
+                         service action))
+          ((_ _) *unspecified*))
+
         (exit 1))
        ((? eof-object?)
         ;; When stopping shepherd, we may get an EOF in lieu of a real reply,
