@@ -44,6 +44,10 @@ cat > "$conf" <<EOF
 		   (system* "$SHELL" "-c" "echo STOPPING")
 		   (delete-file "$stamp"))
 	 #:respawn? #f)
+       (service
+         '(test-system-constructor)
+         #:start (make-system-constructor "date; echo SYSTEM-START")
+         #:stop (make-system-destructor "date; echo SYSTEM-STOP"))
        (service
 	 '(test-command-not-found)
 	 #:start (lambda _
@@ -61,6 +65,10 @@ cat > "$conf" <<EOF
 			  (delete-file "$stamp")
 			  #f)))
 	 #:respawn? #t)))
+
+;; Start this one upfront.  This ensures signal handling and the process
+;; monitor are working as expected early on.
+(start-service (lookup-service 'test-system-constructor))
 EOF
 
 rm -f "$pid"
@@ -73,6 +81,13 @@ shepherd_pid="`cat $pid`"
 
 kill -0 $shepherd_pid
 
+# Did the 'make-system-destructor' start?
+until $herd start test-system-constructor | grep running; do sleep 0.3; done
+grep SYSTEM-START "$log"
+$herd stop test-system-constructor
+$herd status test-system-constructor | grep stopped
+grep SYSTEM-STOP "$log"
+
 # 'herd start' will block until the script exits...
 $herd start test &
 
