@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test transient services.
-# Copyright © 2022 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022, 2023 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -31,14 +31,14 @@ trap "cat $log || true; rm -f $socket $conf $log;
 
 cat > "$conf"<<EOF
 (register-services
- (make <service>
-   #:provides '(transient-test1)
-   #:start (make-forkexec-constructor '("sleep" "600"))
-   #:transient? #t)
- (make <service>
-   #:provides '(transient-test2)
-   #:start (make-forkexec-constructor '("sleep" "600"))
-   #:transient? #t))
+ (list (service
+	 '(transient-test1)
+	 #:start (make-forkexec-constructor '("sleep" "600"))
+	 #:transient? #t)
+       (service
+	 '(transient-test2)
+	 #:start (make-forkexec-constructor '("sleep" "600"))
+	 #:transient? #t)))
 EOF
 
 rm -f "$pid"
@@ -53,13 +53,21 @@ $herd start transient-test1
 $herd start transient-test2
 
 # Stop the service and make sure it gets unregistered.
-$herd status transient-test1 | grep "started and transient"
+$herd status transient-test1 | grep "transient, running"
 $herd stop transient-test1
-! $herd status transient-test1
+$herd status transient-test1 && false
 
 # Terminate the service and make sure it gets unregistered.
-$herd status transient-test2 | grep "started and transient"
+$herd status transient-test2 | grep "transient, running"
 kill $($herd status transient-test2 | grep Running | sed -e's/^.* \([0-9]\+\).*$/\1/g')
-! $herd status transient-test2
+$herd status transient-test2 && false
 
 test $($herd status | grep transient-test | wc -l) -eq 0
+
+# Ensure that restarting a transient service is impossible.
+$herd load root "$conf"
+$herd start transient-test1
+$herd restart transient-test1
+$herd status transient-test1 && false
+
+$herd unload root transient-test2
